﻿using DTOs;
using Microsoft.AspNetCore.Mvc;
using OrderCriterias;

namespace WebApi.Controllers
{
    public class PlayerController : ControllerBase
    {
        private Unit unity;

        private readonly Logger<PlayerController> logger;

        public PlayerController(Unit unit, Logger<PlayerController> logger)
        {
            this.unity = unit;
            this.logger = logger;
        }

        /// <summary>
        /// add a player
        /// </summary>
        /// <param name="player">the player to add</param>
        /// <returns>
        /// status code :
        /// 200 if the player is added
        /// 202 if the player is added
        /// 208 if the player was already in the database
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// no content when status code = 200
        /// player added when status code = 202
        /// player already in the database that equals the one we wanted to add when status code = 208
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> PostPlayer(PlayerDto player)
        {
            int count = unity.getNbPlayers(); // count : number of elements before opperation
            var tmp = await unity.addPlayer(player); //  tmp : player recieve by the addPlayer method
            if (unity.getNbPlayers() == count) // <=> not added
            {
                if (tmp.Nickname == player.Nickname)
                {
                    // it was already in the database
                    logger.LogInformation(message: $"want to add a player already in the database : {tmp.ToString()}");
                    return StatusCode(208, tmp);
                }
                else
                {
                    // we recieve a player already in the database
                    // that should be equal to the player that we
                    // wanted to add but it isn't
                    logger.LogCritical(message: "controller's add fail (already in the database) but\nplayer" +
                        $" recieve isn't the same as the one we wanted to add.\n" +
                        $"Player recieve : {tmp.ToString()}\nplayer to add : {player.ToString()}");
                    return StatusCode(500);
                }
            }
            // added
            if (tmp.Nickname == player.Nickname)
            {
                logger.LogTrace(message: $"player added : {tmp.ToString()}");
                // the player has been added and we recieved him
                return StatusCode(202, tmp);
            }
            else
            {
                // the player may be added but we do not recieved him
                try
                {
                    if (unity.getPlayer(player.Nickname) != null)
                    {
                        // he is added
                        logger.LogError(message: $"player added but not recieved");
                        return Ok(); // not 202 to make a difference between 2 cases
                    }
                    else
                    {
                        // he is not added
                        logger.LogCritical(message: "player that we wanted to add not added\nand we have added another one");
                        if (unity.getPlayer(player.Nickname) == null) // <=> not added
                            return StatusCode(500);
                    }
                }
                catch (Exception)
                {
                    return StatusCode(500);
                }
            }
            logger.LogError(message: "this case should not append");
            return StatusCode(500);
        }

        /// <summary>
        /// get a part of all players
        /// </summary>
        /// <param name="page">the actual page</param>
        /// <param name="count">number of T element in a page</param>
        /// <param name="orderCriteria">the order criteria</param>
        /// <returns>
        /// status code :
        /// 200 if we got a set
        /// 204 if we got an empty set
        /// 400 if we got nothing
        /// 
        /// return content :
        /// a tuple of the number of page and
        /// all players in the database for
        /// the page nb if status code = 200
        /// the number of page for count element if status code = 204
        /// nothing if status code = 400
        /// </returns>
        public async Task<IActionResult> GetSomePlayers(int page, int count = 10, PlayerOrderCriteria orderCriteria = PlayerOrderCriteria.ById)
        {
            var tmp = await unity.getPlayers(page, count, orderCriteria);
            if (tmp.players == null)
            {
                logger.LogInformation(message: "get player : bad request (page or/and count incorrect)");
                return BadRequest(tmp.nbPages);
            }
            else if (tmp.players.Count() == 0)
            {
                logger.LogWarning(message: $"get player : no content. number of element : {unity.getNbPlayers()}, page wanted : {page}, number of elements in a page : {count}");
                return NoContent();
            }
            else
            {
                logger.LogTrace(message: $"get players : page = {page}, count = {count}, order criteria = {orderCriteria switch
                {
                    PlayerOrderCriteria.ById => "byId",
                    PlayerOrderCriteria.ByNickname => "byNickname",
                    _ => "none"
                }}");
                return Ok(tmp);
            }
        }

        /// <summary>
        /// delete a player
        /// </summary>
        /// <param name="id">the id of the player to delete</param>
        /// <returns>
        /// <returns>
        /// status code :
        /// 200 if the player is removed
        /// 204 if the player is removed
        /// 400 if the id is incorrect
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// aministrator deleted when status code = 200
        /// no content when status code = 204
        /// no content when status code = 400
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> DeletePlayer(int id)
        {
            if (id < 0)
            {
                logger.LogError("want to delete a player with an id less than 0");
                return BadRequest();
            }
            int count = unity.getNbPlayers(); // count : number of elements before opperation
            var tmp = await unity.removePlayer(id);
            if (tmp == null) // we don't recieve the player
            {
                if (unity.getNbPlayers() == count) // he is not deleted 
                {
                    if (unity.getPlayer(id) != null)
                    {
                        logger.LogCritical(message: "remove player fail : player not removed and not recieved");
                        return StatusCode(500);
                    }
                    else
                    {
                        logger.LogInformation(message: "trying to remove a player with an id who don't exist");
                        return BadRequest();
                    }
                }
                else // he may be deleted
                {
                    if (unity.getPlayer(id) == null) // he must be deleted
                    {
                        logger.LogError(message: "player removed but not returned");
                        return NoContent();
                    }
                    else // he is not deleted
                    {
                        logger.LogCritical(message: "remove player fail : player to remove not remove\ninstead, anotherone is deleted and we recieved nothing");
                        return StatusCode(500);
                    }
                }
            }
            if (unity.getNbPlayers() == count)
            {
                // <=> we have recieved a player which should be deleted
                // but since we have the same number of player than
                // before deletion, it isn't deleted
                logger.LogCritical(message: $"player \"{tmp.ToString()}\"should be delete but it isn't");
                return StatusCode(500);
            }
            logger.LogTrace(message: $"player removed {tmp.ToString()}");
            return Ok(tmp);
        }

        /// <summary>
        /// get a player
        /// </summary>
        /// <param name="id">the id of the player</param>
        /// <returns>
        /// status code :
        /// 200 when we got a player
        /// 204 when we got null
        /// 
        /// return content :
        /// the player that correspond to the id when status code = 200
        /// nothing when status code = 204
        /// </returns>
        public async Task<IActionResult> GetPlayer(int id)
        {
            var tmp = await unity.getPlayer(id);
            if (tmp == null) return NoContent();
            return Ok(tmp);
        }

        /// <summary>
        /// update a player
        /// </summary>
        /// <param name="id">the id of the player</param>
        /// <returns>
        /// status code :
        /// 200 when we got a player
        /// 204 when we got null
        /// 208 when the nickname is already used
        /// return content :
        /// the player that correspond to the id when status code = 200
        /// nothing when status code = 204
        /// the player which have this nickname when status code = 208
        /// </returns>
        public async Task<IActionResult> PutPlayer(int id, PlayerDto player)
        {
            var tmp = await unity.getPlayer(player.Nickname);
            if (tmp != null) return StatusCode(208);
            tmp = await unity.updatePlayer(id, player);
            if (tmp == null) return NoContent();
            return Ok(tmp);
        }
    }
}
