﻿using DTOs;
using Microsoft.AspNetCore.Mvc;
using OrderCriterias;

namespace WebApi.Controllers
{
    public class QuestionController : ControllerBase
    {
        private Unit unity;

        private readonly Logger<QuestionController> logger;

        public QuestionController(Unit unit, Logger<QuestionController> logger)
        {
            this.unity = unit;
            this.logger = logger;
        }

        /// <summary>
        /// add a question
        /// </summary>
        /// <param name="question">the question to add</param>
        /// <returns>
        /// status code :
        /// 200 if the question is added
        /// 202 if the question is added
        /// 208 if the question was already in the database
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// no content when status code = 200
        /// question added when status code = 202
        /// question already in the database that equals the one we wanted to add when status code = 208
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> PostQuestion(QuestionDto question)
        {
            int count = unity.getNbQuestions(); // count : number of elements before opperation
            var tmp = await unity.addQuestion(question); //  tmp : question recieve by the addQuestion method
            if (unity.getNbQuestions() == count) // <=> not added
            {
                if (tmp.Content == question.Content)
                {
                    // it was already in the database
                    logger.LogInformation(message: $"want to add a question already in the database : {tmp.ToString()}");
                    return StatusCode(208, tmp);
                }
                else
                {
                    // we recieve a question already in the database
                    // that should be equal to the question that we
                    // wanted to add but it isn't
                    logger.LogCritical(message: "controller's add fail (already in the database) but\nquestion" +
                        $" recieve isn't the same as the one we wanted to add.\n" +
                        $"Question recieve : {tmp.ToString()}\nquestion to add : {question.ToString()}");
                    return StatusCode(500);
                }
            }
            // added
            if (tmp.Content == question.Content)
            {
                logger.LogTrace(message: $"question added : {tmp.ToString()}");
                // the question has been added and we recieved him
                return StatusCode(202, tmp);
            }
            else
            {
                // the question may be added but we do not recieved him
                try
                {
                    if (unity.getQuestion(question.Content) != null)
                    {
                        // he is added
                        logger.LogError(message: $"question added but not recieved");
                        return Ok(); // not 202 to make a difference between 2 cases
                    }
                    else
                    {
                        // he is not added
                        logger.LogCritical(message: "question that we wanted to add not added\nand we have added another one");
                        if (unity.getQuestion(question.Content) == null) // <=> not added
                            return StatusCode(500);
                    }
                }
                catch (Exception)
                {
                    return StatusCode(500);
                }
            }
            logger.LogError(message: "this case should not append");
            return StatusCode(500);
        }

        /// <summary>
        /// get a part of all questions
        /// </summary>
        /// <param name="page">the actual page</param>
        /// <param name="count">number of T element in a page</param>
        /// <param name="orderCriteria">the order criteria</param>
        /// <returns>
        /// status code :
        /// 200 if we got a set
        /// 204 if we got an empty set
        /// 400 if we got nothing
        /// 
        /// return content :
        /// a tuple of the number of page and
        /// all questions in the database for
        /// the page nb if status code = 200
        /// the number of page for count element if status code = 204
        /// nothing if status code = 400
        /// </returns>
        public async Task<IActionResult> GetSomeQuestions(int page, int count = 10, QuestionOrderCriteria orderCriteria = QuestionOrderCriteria.ById)
        {
            var tmp = await unity.getQuestions(page, count, orderCriteria);
            if (tmp.questions == null)
            {
                logger.LogInformation(message: "get question : bad request (page or/and count incorrect)");
                return BadRequest(tmp.nbPages);
            }
            else if (tmp.questions.Count() == 0)
            {
                logger.LogWarning(message: $"get question : no content. number of element : {unity.getNbQuestions()}, page wanted : {page}, number of elements in a page : {count}");
                return NoContent();
            }
            else
            {
                logger.LogTrace(message: $"get questions : page = {page}, count = {count}, order criteria = {orderCriteria switch
                {
                    QuestionOrderCriteria.ById => "byId",
                    QuestionOrderCriteria.ByContent => "byContent",
                    QuestionOrderCriteria.ByIdChapter => "byIdChapter",
                    QuestionOrderCriteria.ByNbFalls => "byNbFalls",
                    QuestionOrderCriteria.ByDifficulty => "byDifficulty",
                    _ => "none"
                }}");
                return Ok(tmp);
            }
        }

        /// <summary>
        /// delete a question
        /// </summary>
        /// <param name="id">the id of the question to delete</param>
        /// <returns>
        /// status code :
        /// 200 if the question is removed
        /// 204 if the question is removed
        /// 400 if the id is incorrect
        /// 500 if there was an internal error that doesn't allow to continue
        /// return content :
        /// aministrator deleted when status code = 200
        /// no content when status code = 204
        /// no content when status code = 400
        /// no content when status code = 500
        /// </returns>
        public async Task<IActionResult> DeleteQuestion(int id)
        {
            if (id < 0)
            {
                logger.LogError("want to delete a question with an id less than 0");
                return BadRequest();
            }
            int count = unity.getNbQuestions(); // count : number of elements before opperation
            var tmp = await unity.removeQuestion(id);
            if (tmp == null) // we don't recieve the question
            {
                if (unity.getNbQuestions() == count) // he is not deleted 
                {
                    if (unity.getQuestion(id) != null)
                    {
                        logger.LogCritical(message: "remove question fail : question not removed and not recieved");
                        return StatusCode(500);
                    }
                    else
                    {
                        logger.LogInformation(message: "trying to remove a question with an id who don't exist");
                        return BadRequest();
                    }
                }
                else // he may be deleted
                {
                    if (unity.getQuestion(id) == null) // he must be deleted
                    {
                        logger.LogError(message: "question removed but not returned");
                        return NoContent();
                    }
                    else // he is not deleted
                    {
                        logger.LogCritical(message: "remove question fail : question to remove not remove\ninstead, anotherone is deleted and we recieved nothing");
                        return StatusCode(500);
                    }
                }
            }
            if (unity.getNbQuestions() == count)
            {
                // <=> we have recieved a question which should be deleted
                // but since we have the same number of question than
                // before deletion, it isn't deleted
                logger.LogCritical(message: $"question \"{tmp.ToString()}\"should be delete but it isn't");
                return StatusCode(500);
            }
            logger.LogTrace(message: $"question removed {tmp.ToString()}");
            return Ok(tmp);
        }

        /// <summary>
        /// get a question
        /// </summary>
        /// <param name="id">the id of the question</param>
        /// <returns>
        /// status code :
        /// 200 when we got a question
        /// 204 when we got null
        /// 
        /// return content :
        /// the question that correspond to the id when status code = 200
        /// nothing when status code = 204
        /// </returns>
        public async Task<IActionResult> GetQuestion(int id)
        {
            var tmp = await unity.getQuestion(id);
            if (tmp == null) return NoContent();
            return Ok(tmp);
        }

        /// <summary>
        /// update a question
        /// </summary>
        /// <param name="id">the id of the question</param>
        /// <returns>
        /// status code :
        /// 200 when we got a question
        /// 204 when we got null
        /// 208 when the nickname is already used
        /// return content :
        /// the question that correspond to the id when status code = 200
        /// nothing when status code = 204
        /// the question which have this nickname when status code = 208
        /// </returns>
        public async Task<IActionResult> PutQuestion(int id, QuestionDto question)
        {
            var tmp = await unity.getQuestion(question.Content);
            if (tmp != null) return StatusCode(208);
            tmp = await unity.updateQuestion(id, question);
            if (tmp == null) return NoContent();
            return Ok(tmp);
        }
    }
}
